/**
 * Created by Brian on 20/09/2016.
 */
var expanded = {};
var l10n_strings = {};
var delay = 100;
var default_refresh_src = "../images/refresh-preview.svg";

var sections = [
    {
        name: 'Imagery',
        localizedName: l10n("Imagery"),
        localizedTitle: l10n("Import Map Tiles"),
        icon: 'imagery.svg'
    },
    {
        name: 'Roads',
        localizedName: l10n("Roads"),
        localizedTitle: l10n("Import OSM Road Data"),
        icon: 'roads.svg'
    },
    {
        name: 'Paths',
        localizedName: l10n("Paths"),
        localizedTitle: l10n("Import OSM Path Data"),
        icon: 'paths.svg'
    },
    {
        name: 'Buildings',
        localizedName: l10n("Buildings Global"),
        localizedTitle: l10n("Import OSM Buildings - Start Here for all countries including USA"),
        icon: 'buildings.svg'
    },
    {
        name: 'Water',
        localizedName: l10n("Water"),
        localizedTitle: l10n("Import OSM Water Bodies"),
        icon: 'water.svg'
    },
    {
        name: 'Trees',
        localizedName: l10n("Trees"),
        localizedTitle: l10n("Import OSM Tree Locations"),
        icon: 'tree.svg'
    }
];

var im_section = false;
var import_hash = {};

function addHeader(){
    document.write(PlaceMaker.header());
}

function addImportActions(){
    document.write(PlaceMaker.importActions());
    sections.forEach(add_section_to_html);
    addMSBuildingsImportAction();
}

// Since this action doesn't follow our standard soy template scheme like the other options
// we'll use JS to dynamically insert the HTML and event handling here.
function addMSBuildingsImportAction() {
    var button = document.createElement("BUTTON");
    var buttonImage = document.createElement("IMG");
    var labelDiv =  document.createElement("DIV");
    var title = l10n("Import Microsoft Buildings - Fill in missing buildings for entire USA");

    buttonImage.src = "../images/buildings-usa.svg";
    button.classList.add('import-button');
    button.classList.add('second-row');
    button.title = title;
    labelDiv.classList.add('import-action-label');
    labelDiv.classList.add('second-row');
    labelDiv.title = title;
    labelDiv.innerHTML = 'Buildings USA (Beta)';

    button.appendChild(buttonImage);

    var el = document.querySelector(".Buildings-section .main-options");
    el.appendChild(button);

    el = document.querySelector(".Buildings-section .main-options .expandable-container");
    el.appendChild(labelDiv);

    button.addEventListener('click', importFeature);
    $(button).data('name', 'Buildings'); //key, value
    $(button).data('usaBuildings', true); //key, value

    // Align the status icons and chevron to be in between the two buildings rows
    $(".Buildings-section .status").css("top", "25px");
    $(".Buildings-section .expand-switch").css("top", "25px");
}

function addSubSections() {
    window.location = 'skp:load_prefs_and_listeners';
}

function expand(section) {
    toggleVisible(section);
    toggleChevronRotation(section);
}

function toggle3DOptionsVisible() {
    var options = document.querySelector('.volume-building-options');
    toggleVisible(options);
}

function toggleHeightOptionsVisible() {
    var options = document.querySelector('.random-height-options');
    toggleVisible(options);
}

function toggleVisible(node) {
    $(node).slideToggle();
}

function toggleChevronRotation(section) {
    var importSection = section.parentNode;

    var chevron = importSection.querySelector('.expand-switch');
    var classString = chevron.className;

    if (classString.search(/(^| )expanded($| )/) != -1) {
        classString = classString.replace(/(^| )expanded($| )/, '');
        chevron.className = classString;
    } else {
        classString += ' expanded';
        chevron.className = classString;
    }
}

function setStatus(sectionName, status, message) {
    var section = getImportSection(sectionName);
    var images = section.querySelectorAll('.status img');
    for (var i = 0; i < images.length; ++i) {
        hideElement(images[i]);
    }
    var statusImage;
    switch (status) {
        case 'waiting':
            statusImage = section.querySelector('.wait img');
            break;
        case 'done':
            statusImage = section.querySelector('.done img');
            break;
        case 'error':
            statusImage = section.querySelector('.error img');
            break;
    }
    showElement(statusImage);
    statusImage.title = message;
}

function resetStatus(sectionName) {
    var section = getImportSection(sectionName);
    var statusImage = section.querySelector('.wait img');
    hideElement(statusImage);
    statusImage = section.querySelector('.done img');
    hideElement(statusImage);
    statusImage = section.querySelector('.error img');
    hideElement(statusImage);
}

function showErrorMessage(node, message) {
    node.innerHTML = message;
}


function collapseAll() {
    var containers = document.getElementsByClassName('options-container');
    var nodeArray = Array.apply(null, containers);
    nodeArray.forEach(function(container) {
        hideElement(container);
    });
}

//TODO can this be refactord since most are identical appart from the section name!
function addListeners() {

    //Imagery section preveiew pane
    document.querySelector('.preview-pane').
    addEventListener('click', updatePreview);

    sections.forEach(function(value) {
        var import_name = value.name;
        var importButton = document.querySelector('.' + import_name + '-section .import-button');
        $(importButton).data('name', import_name); //needs to be a jquery element

        importButton.addEventListener('click', importFeature);
        var importOptions = document.querySelector('.' + import_name + '-section .options-container');
        document.querySelector('.' + import_name + '-section .expandable-container').
            addEventListener('click', expand.bind(this, importOptions));
    });

    var importAllButton = document.querySelector('.import-all-button');
    importAllButton.addEventListener('click', importAll.bind(this));

    var saveSettingsButton = document.querySelector('.save-preferences-link');
    saveSettingsButton.addEventListener('click', saveSettings.bind(this));

    var geolocateButton = document.querySelector('.geolocate-button');
    geolocateButton.addEventListener('click', openGeoWindow.bind(this));

    var treeLibraryButton = document.querySelector('.tree-library-button');
    treeLibraryButton.addEventListener('click', selectTreeLibrary.bind(this));

    var make3DCheckbox = document.querySelector('.make3D');
    make3DCheckbox.addEventListener('click', toggle3DOptionsVisible.bind(this));

    var randomHeightCheckbox = document.querySelector('.randomize-height');
    randomHeightCheckbox.addEventListener('click', toggleHeightOptionsVisible.bind(this));

    var osmTerms = document.querySelector('.osm-terms-link');
    osmTerms.addEventListener('click', openTerms.bind(this, 'osm'));

    var drapeRoads = document.querySelector('.drape-roads');
    drapeRoads.addEventListener('click', validateDrape.bind(this, 'drape-roads'));

    var mergePaths = document.querySelector('.merge-paths');
    mergePaths.addEventListener('click', validateDrape.bind(this, 'merge-paths'));

    var dropBuildings = document.querySelector('.drop-buildings');
    dropBuildings.addEventListener('click', validateDrape.bind(this, 'drop-buildings'));

    var dropTrees = document.querySelector('.drop-trees');
    dropTrees.addEventListener('click', validateDrape.bind(this, 'drop-trees'));

    var dropWater = document.querySelector('.merge-water');
    dropWater.addEventListener('click', validateDrape.bind(this, 'merge-water'));

    var imagerySource = document.querySelector('.Imagery-section .source');
    imagerySource.addEventListener('change', updateImagerySection.bind(this));
}

//todo this can be moved to the bottom
function buyMoreBuildings() {
    window.location = 'skp:buyMoreBuildings';
}

//This function will update the imagery sub options based on source selected
function updateImagerySection() {
    var imagerySource = document.querySelector('.Imagery-section .source');
    var mapChoice = document.querySelector('.map-type-row');
    if (imagerySource.value == 'Nearmap') {
        mapChoice.style.display = 'none';
    } else {
        mapChoice.style.display = 'block'
    }
    var preview = document.querySelector('.preview-pane');
    enableElement(preview);
    var icon = preview.querySelector('.refresh-icon');
    showElement(icon);
    hideElement(preview.querySelector('.spinner'));
    icon.src = default_refresh_src;
}

//might display the merge/drape import time warning to user
function validateDrape(className) {
    window.location = 'skp:validate_drape@' + className;
}

//update the imagery section preview image
function updatePreview() {
    var preview = document.querySelector('.preview-pane');
    disableElement(preview);
    hideElement(preview.querySelector('.refresh-icon'));
    showElement(preview.querySelector('.spinner'));
    options = getImageryOptions();
    setTimeout(function() {
        window.location = 'skp:update_preview@' + JSON.stringify(options);
    }, delay);
}

//rest imagery section preview
function resetPreview() {
    var preview = document.querySelector('.preview-pane');
    disableElement(preview);
    showElement(preview.querySelector('.refresh-icon'));
    hideElement(preview.querySelector('.spinner'));
}

//set imagery section preview
function setImagePreview(src) {
    var preview = document.querySelector('.preview-pane');
    enableElement(preview);
    var icon = preview.querySelector('.refresh-icon');
    showElement(icon);
    hideElement(preview.querySelector('.spinner'));
    icon.src = src;
}

//Called by the Make Place button -- does not import Imagery or Terrain
//TODO Edit when adding an import section!
function importAll() {
    var importAllButton = document.querySelector('.import-all-button');
    disableButtons(importAllButton);
    var allOptions = {
        Buildings: getBuildingsOptions(),
        Roads: getRoadsOptions(),
        Trees: getTreesOptions(),
        Water: getWaterOptions(),
        Paths: getPathsOptions()
    };

    setTimeout(function() {
        window.location = 'skp:import_all@' + JSON.stringify(allOptions);
    }, delay);
}

//Save all import settins
//TODO Edit when adding an import section!
function saveSettings() {
    var allOptions = {
        Imagery: getImageryOptions(),
        Buildings: getBuildingsOptions(),
        Roads: getRoadsOptions(),
        Trees: getTreesOptions(),
        Water: getWaterOptions(),
        Paths: getPathsOptions()
    };
    setTimeout(function() {
        window.location = 'skp:save_as_defaults@' + JSON.stringify(allOptions);
    }, delay);
}

//open select place
function openGeoWindow() {
    window.location = 'skp:open_geo_window';
}

function validateSurface() {
    window.location = 'skp:validate_surface';
}

function selectTreeLibrary() {
    window.location = 'skp:select_tree_library';
}

function setTreeLibrary(text) {
    var treeLibraryInput = document.querySelector('.tree-library input');
    treeLibraryInput.value = text;
}

function importFeature(evt) {
    //target should be the element clicked
    //currentTarget should be the element the event listener is attached
    //.target not working on mac --- currentTarget appears to work
    var elem = $(evt.currentTarget); //jquery elem
    var name = elem.data('name');

    // We include a slight delay to allow for manipulating and refreshing of the DOM
    setTimeout(function() {
        var options = getImportOptions(name);
        if (elem.data('usaBuildings') == true)  {
            options.options.source = 'MS';
            if (typeof esriBuildingsURL !== 'undefined') {
                options.options.server = esriBuildingsURL; // defined remotely by
            }
        }
        window.location = 'skp:import_type@' + JSON.stringify(options);
    }, delay);   
}

//prevent the user from starting the same operation that is already started
function startImport(sectionName) {
    var importButtons = getImportButtons(sectionName);
    disableButtons(importButtons);
    setStatus(sectionName, 'waiting', '');
}

//re-enable the import button
function resetImport(sectionName) {
    resetStatus(sectionName);
    var importButtons = getImportButtons(sectionName);
    enableButtons(importButtons);
}

//re-enable Make Place button
function resetImportAll() {
    enableButtons(document.querySelector('.import-all-button'));
}

function displayImportError(sectionName, message) {
    setStatus(sectionName, 'error', message);
    enableButtons(getImportButtons(sectionName));
}

function importComplete(sectionName, message) {
    var importButton = getImportButtons(sectionName);
    enableButtons(importButton);
    setStatus(sectionName, 'done', message);
}

function getImportButtons(sectionName) {
    var section = getImportSection(sectionName);
    return section.querySelectorAll('.import-button');
}

function getImportSection(sectionName) {
    return document.querySelector('.' + sectionName + '-section');
}

//todo is there a way to refactor this so the option name is extracted and the class name has a genaric format that indicates its an option
function getImportOptions(name) {
    switch(name) {
        case 'Roads':
            import_options = getRoadsOptions();
            break;
        case 'Buildings':
            import_options = getBuildingsOptions();
            break;
        case 'Water':
            import_options = getWaterOptions();
            break;
        case 'Paths':
            import_options = getPathsOptions();
            break;
        case 'Trees':
            import_options = getTreesOptions();
            break;
        case 'Imagery':
            import_options = getImageryOptions();
            break;
        default:
            import_options = {};
    }

    option_hash = {
        "type": name,
        "options": import_options
    };

    return option_hash;
}

function getImageryOptions() {
    options = {
        source: document.querySelector('.Imagery-section .source').value,
        zoom: document.getElementsByClassName('zoom')[0].value,
        materialEnabled: document.getElementsByClassName('paint-selected')[0].checked,
        mapType: document.getElementsByClassName('map-type')[0].value
    };
    return options;
}

function getRoadsOptions() {
    var options = {
        defaultWidth: encode(document.querySelector('.road-width-input input').value),
        serviceWidth: encode(document.querySelector('.service-road-width-input input').value),
        residentialWidth: encode(document.querySelector('.residential-road-width-input input').value),
        highwayWidth: encode(document.querySelector('.highway-road-width-input input').value),
        motorwayWidth: encode(document.querySelector('.motorway-road-width-input input').value),
        laneWidth: encode(document.querySelector('.lane-width-input input').value),
        drape: document.querySelector('.drape-roads').checked
    };
    return options;
}

function getPathsOptions() {
    var options = {
        defaultWidth: encode(document.querySelector('.path-width-input input').value),
        drape: document.querySelector('.merge-paths').checked
    };
    return options;
}

function getWaterOptions() {
    var options = {
        drape: document.querySelector('.merge-water').checked
    };
    return options;
}

function getTreesOptions() {
    var options = {
        library: encode(document.querySelector('.Trees-section .tree-library input').value),
        minHeight: encode(document.querySelector('.min-tree-height input').value),
        maxHeight: encode(document.querySelector('.max-tree-height input').value),
        spacing: encode(document.querySelector('.tree-spacing input').value),
        drape: document.querySelector('.drop-trees').checked
    };
    return options;
}

function getBuildingsOptions() {
    var options = {
        defaultHeight: encode(document.querySelector('.building-height-input input').value),
        levelHeight: encode(document.querySelector('.level-height-input input').value),
        make3D: document.querySelector('.make3D').checked,
        randomHeight: document.querySelector('.randomize-height').checked,
        materialEnabled: document.querySelector('.import-building-material').checked,
        drape: document.querySelector('.drop-buildings').checked,
        minHeight: encode(document.querySelector('.min-height-input input').value),
        maxHeight: encode(document.querySelector('.max-height-input input').value),
        autoAddRoofs: document.querySelector('.auto-add-roofs').checked
    };
    return options;
}


//todo can these add{type}Options be refactored?
// function addImportOptions(import_name, defaults) {
//     var importElement = document.querySelector('.' + import_name + '-section');
//     var importOptions = goog.soy.renderAsFragment(PlaceMaker.terrain, defaults);
//     importElement.appendChild(importOptions);  
// }

function addImageryOptions(defaults) {
    var imagery = document.querySelector('.Imagery-section');
    var imageryOptions = goog.soy.renderAsFragment(PlaceMaker.imagery, defaults);
    imagery.appendChild(imageryOptions);
}

function addRoadsOptions(defaults) {
    var roads = document.querySelector('.Roads-section');
    var options = goog.soy.renderAsFragment(PlaceMaker.roads, defaults);
    roads.appendChild(options);
}

function addPathsOptions(defaults) {
    var paths = document.querySelector('.Paths-section');
    var options = goog.soy.renderAsFragment(PlaceMaker.paths, defaults);
    paths.appendChild(options);
}

function addBuildingsOptions(defaults) {
    var section = document.querySelector('.Buildings-section');
    var options = goog.soy.renderAsFragment(PlaceMaker.buildings, defaults);
    section.appendChild(options);
}

function addTreesOptions(defaults) {
    var section = document.querySelector('.Trees-section');
    var options = goog.soy.renderAsFragment(PlaceMaker.trees, defaults);
    section.appendChild(options);
}

function addWaterOptions(defaults) {
    var section = document.querySelector('.Water-section');
    var options = goog.soy.renderAsFragment(PlaceMaker.water, defaults);
    section.appendChild(options);
}

function openTerms(type) {
    window.location = 'skp:open_terms@' + type;
}

function add_section_to_html(value, index) {
    document.write(PlaceMaker.importSection(value));
}

function addAttribution() {
    document.write(PlaceMaker.attribution());
}


